# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from airflow import DAG
from airflow.providers.google.cloud.operators import kubernetes_engine

default_args = {
    "owner": "Google",
    "depends_on_past": False,
    "start_date": "2022-06-30",
}


with DAG(
    dag_id="census_bureau_acs.census_bureau_acs",
    default_args=default_args,
    max_active_runs=1,
    schedule_interval="@monthly",
    catchup=False,
    default_view="graph",
) as dag:
    create_cluster = kubernetes_engine.GKECreateClusterOperator(
        task_id="create_cluster",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        body={
            "name": "census-bureau-acs",
            "initial_node_count": 30,
            "network": "{{ var.value.vpc_network }}",
            "node_config": {
                "machine_type": "e2-standard-4",
                "oauth_scopes": [
                    "https://www.googleapis.com/auth/devstorage.read_write",
                    "https://www.googleapis.com/auth/cloud-platform",
                ],
            },
        },
    )

    # Run CSV transform within kubernetes pod
    cbsa_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="cbsa_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "cbsa_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "metropolitan%20statistical%20area/micropolitan%20statistical%20area",
            "GEOGRAPHY": "cbsa",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["combined_statistical_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"combined_statistical_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    cbsa_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="cbsa_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.cbsa_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.cbsa_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.cbsa_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.cbsa_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.cbsa_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "cbsa_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.cbsa_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "metropolitan%20statistical%20area/micropolitan%20statistical%20area",
            "GEOGRAPHY": "cbsa",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["combined_statistical_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"combined_statistical_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    cbsa_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="cbsa_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "cbsa_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "metropolitan%20statistical%20area/micropolitan%20statistical%20area",
            "GEOGRAPHY": "cbsa",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["combined_statistical_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"combined_statistical_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    cbsa_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="cbsa_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.cbsa_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.cbsa_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.cbsa_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.cbsa_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.cbsa_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "cbsa_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "metropolitan%20statistical%20area/micropolitan%20statistical%20area",
            "GEOGRAPHY": "cbsa",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["combined_statistical_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"combined_statistical_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    cbsa_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="cbsa_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.cbsa_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.cbsa_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.cbsa_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.cbsa_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.cbsa_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "cbsa_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.cbsa_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "metropolitan%20statistical%20area/micropolitan%20statistical%20area",
            "GEOGRAPHY": "cbsa",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["combined_statistical_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"combined_statistical_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    cbsa_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="cbsa_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.cbsa_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.cbsa_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.cbsa_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.cbsa_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.cbsa_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "cbsa_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.cbsa_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "metropolitan%20statistical%20area/micropolitan%20statistical%20area",
            "GEOGRAPHY": "cbsa",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["combined_statistical_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"combined_statistical_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    censustract_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="censustract_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.censustract_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.censustract_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.censustract_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.censustract_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.censustract_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "censustract_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.censustract_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "tract",
            "GEOGRAPHY": "censustract",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","county","tract"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county","4":"tract" }',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "4", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "12G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    censustract_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="censustract_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.censustract_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.censustract_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.censustract_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.censustract_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.censustract_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "censustract_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.censustract_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "tract",
            "GEOGRAPHY": "censustract",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","county","tract"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county","4":"tract" }',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "4", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "12G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    censustract_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="censustract_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.censustract_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.censustract_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.censustract_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.censustract_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.censustract_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "censustract_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.censustract_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "tract",
            "GEOGRAPHY": "censustract",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","county","tract"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county","4":"tract" }',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "4", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "12G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    congressionaldistrict_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="congressionaldistrict_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "congressionaldistrict_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "congressional%20district",
            "GEOGRAPHY": "congressionaldistrict",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","congressional_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"congressional_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    congressionaldistrict_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="congressionaldistrict_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "congressionaldistrict_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "congressional%20district",
            "GEOGRAPHY": "congressionaldistrict",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","congressional_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"congressional_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    congressionaldistrict_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="congressionaldistrict_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "congressionaldistrict_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "congressional%20district",
            "GEOGRAPHY": "congressionaldistrict",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","congressional_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"congressional_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    congressionaldistrict_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="congressionaldistrict_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "congressionaldistrict_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.congressionaldistrict_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "congressional%20district",
            "GEOGRAPHY": "congressionaldistrict",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","congressional_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"congressional_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    congressionaldistrict_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="congressionaldistrict_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "congressionaldistrict_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.congressionaldistrict_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "congressional%20district",
            "GEOGRAPHY": "congressionaldistrict",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","congressional_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"congressional_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    congressionaldistrict_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="congressionaldistrict_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "congressionaldistrict_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.congressionaldistrict_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "congressional%20district",
            "GEOGRAPHY": "congressionaldistrict",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","congressional_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"congressional_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    county_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="county_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.county_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.county_2019_1yr.target_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.county_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.county_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.county_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "county_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.county_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "county",
            "GEOGRAPHY": "county",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","county"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    county_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="county_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.county_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.county_2020_1yr.target_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.county_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.county_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.county_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "county_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.county_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "county",
            "GEOGRAPHY": "county",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","county"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    county_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="county_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.county_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.county_2021_1yr.target_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.county_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.county_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.county_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "county_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.county_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "county",
            "GEOGRAPHY": "county",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","county"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    county_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="county_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.county_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.county_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.county_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.county_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.county_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "county_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.county_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "county",
            "GEOGRAPHY": "county",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","county"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    county_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="county_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.county_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.county_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.county_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.county_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.county_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "county_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.county_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "county",
            "GEOGRAPHY": "county",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","county"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    county_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="county_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.county_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.county_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.county_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.county_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.county_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "county_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.county_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "county",
            "GEOGRAPHY": "county",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","county"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"county"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    place_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="place_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.place_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.place_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.place_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.place_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.place_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "place_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.place_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "place",
            "GEOGRAPHY": "place",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","place"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"place"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    place_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="place_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.place_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.place_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.place_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.place_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.place_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "place_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.place_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "place",
            "GEOGRAPHY": "place",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","place"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"place"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    place_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="place_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.place_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.place_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.place_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.place_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.place_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "place_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.place_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "place",
            "GEOGRAPHY": "place",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","place"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"place"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    place_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="place_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.place_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.place_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.place_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.place_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.place_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "place_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.place_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "place",
            "GEOGRAPHY": "place",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","place"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"place"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    place_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="place_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.place_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.place_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.place_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.place_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.place_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "place_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.place_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "place",
            "GEOGRAPHY": "place",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","place"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"place"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    place_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="place_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.place_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.place_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.place_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.place_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.place_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "place_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.place_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "place",
            "GEOGRAPHY": "place",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","place"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"place"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    puma_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="puma_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.puma_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.puma_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.puma_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.puma_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.puma_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.puma_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "puma_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.puma_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "public%20use%20microdata%20area",
            "GEOGRAPHY": "puma",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","public_use_microdata_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"public_use_microdata_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    puma_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="puma_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.puma_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.puma_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.puma_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.puma_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.puma_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.puma_2020_1yr.schema_path }}",
            "PIPELINE_NAME": "puma_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.puma_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "public%20use%20microdata%20area",
            "GEOGRAPHY": "puma",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","public_use_microdata_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"public_use_microdata_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    puma_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="puma_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.puma_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.puma_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.puma_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.puma_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.puma_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.puma_2021_1yr.schema_path }}",
            "PIPELINE_NAME": "puma_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.puma_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "public%20use%20microdata%20area",
            "GEOGRAPHY": "puma",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","public_use_microdata_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"public_use_microdata_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    puma_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="puma_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.puma_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.puma_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.puma_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.puma_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.puma_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.puma_2019_5yr.schema_path }}",
            "PIPELINE_NAME": "puma_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.puma_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "public%20use%20microdata%20area",
            "GEOGRAPHY": "puma",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","public_use_microdata_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"public_use_microdata_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    puma_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="puma_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.puma_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.puma_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.puma_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.puma_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.puma_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.puma_2020_5yr.schema_path }}",
            "PIPELINE_NAME": "puma_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.puma_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "public%20use%20microdata%20area",
            "GEOGRAPHY": "puma",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","public_use_microdata_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"public_use_microdata_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    puma_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="puma_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.puma_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.puma_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.puma_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.puma_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.puma_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.puma_2021_5yr.schema_path }}",
            "PIPELINE_NAME": "puma_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.puma_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "public%20use%20microdata%20area",
            "GEOGRAPHY": "puma",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state","public_use_microdata_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"public_use_microdata_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2016_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2016_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2016_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2017_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2017_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2017_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2018_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2018_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2018_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2016_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2016_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2016_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2016_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2017_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2017_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2017_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2017_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2018_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2018_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2018_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2018_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictelementary_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictelementary_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictelementary_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictelementary_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(elementary)",
            "GEOGRAPHY": "schooldistrictelementary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2016_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2016_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2016_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2017_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2017_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2017_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2018_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2018_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2018_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2016_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2016_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2016_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2016_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2017_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2017_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2017_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2017_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2018_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2018_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2018_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2018_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2019_5yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictsecondary_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictsecondary_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictsecondary_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictsecondary_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(secondary)",
            "GEOGRAPHY": "schooldistrictsecondary",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2016_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2016_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2016_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2017_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2017_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2017_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2018_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2018_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2018_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2016_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2016_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2016_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2016_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2017_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2017_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2017_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2017_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2018_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2018_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2018_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2018_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    schooldistrictunified_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="schooldistrictunified_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.schema_path }}",
            "PIPELINE_NAME": "schooldistrictunified_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.schooldistrictunified_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "school%20district%20(unified)",
            "GEOGRAPHY": "schooldistrictunified",
            "REPORT_LEVEL": "state_level",
            "CONCAT_COL_LIST": '["state","school_district"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"school_district"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    state_2019_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="state_2019_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.state_2019_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.state_2019_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.state_2019_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.state_2019_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.state_2019_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "state_2019_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.state_2019_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "state",
            "GEOGRAPHY": "state",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    state_2020_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="state_2020_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.state_2020_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.state_2020_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.state_2020_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.state_2020_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.state_2020_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "state_2020_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.state_2020_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "state",
            "GEOGRAPHY": "state",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    state_2021_1yr = kubernetes_engine.GKEStartPodOperator(
        task_id="state_2021_1yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.state_2021_1yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.state_2021_1yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.state_2021_1yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.state_2021_1yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.state_2021_1yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2021_1yr.schema_path }}",
            "PIPELINE_NAME": "state_2021_1yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.state_2021_1yr.target_gcs_path }}",
            "YEAR_REPORT": "1",
            "API_NAMING_CONVENTION": "state",
            "GEOGRAPHY": "state",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    state_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="state_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.state_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.state_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.state_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.state_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.state_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.state_2019_5yr.schema_path }}",
            "PIPELINE_NAME": "state_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.state_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "state",
            "GEOGRAPHY": "state",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    state_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="state_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.state_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.state_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.state_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.state_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.state_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "state_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.state_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "state",
            "GEOGRAPHY": "state",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    state_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="state_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.state_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.state_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.state_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.state_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.state_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.state_2021_5yr.schema_path }}",
            "PIPELINE_NAME": "state_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.state_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "state",
            "GEOGRAPHY": "state",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["state"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    zcta_2019_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="zcta_2019_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.zcta_2019_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.zcta_2019_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.zcta_2019_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.zcta_2019_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.zcta_2019_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.zcta_2019_5yr.schema_path }}",
            "PIPELINE_NAME": "zcta_2019_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.zcta_2019_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "zip%20code%20tabulation%20area",
            "GEOGRAPHY": "zcta",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["zip_code_tabulation_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"zip_code_tabulation_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    zcta_2020_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="zcta_2020_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.zcta_2020_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.zcta_2020_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.zcta_2020_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.zcta_2020_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.zcta_2020_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.cbsa_2019_1yr.schema_path }}",
            "PIPELINE_NAME": "zcta_2020_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.zcta_2020_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "zip%20code%20tabulation%20area",
            "GEOGRAPHY": "zcta",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["zip_code_tabulation_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"zip_code_tabulation_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )

    # Run CSV transform within kubernetes pod
    zcta_2021_5yr = kubernetes_engine.GKEStartPodOperator(
        task_id="zcta_2021_5yr",
        startup_timeout_seconds=1000,
        name="load_data",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        cluster_name="census-bureau-acs",
        namespace="default",
        image_pull_policy="Always",
        image="{{ var.json.census_bureau_acs.container_registry.run_csv_transform_kub }}",
        env_vars={
            "SOURCE_URL": "{{ var.json.census_bureau_acs.zcta_2021_5yr.source_url }}",
            "SOURCE_FILE": "{{ var.json.census_bureau_acs.zcta_2021_5yr.source_file }}",
            "TARGET_FILE": "{{ var.json.census_bureau_acs.zcta_2021_5yr.target_file }}",
            "CHUNKSIZE": "{{ var.json.census_bureau_acs.chunk_size }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.census_bureau_acs.zcta_2021_5yr.dataset_id }}",
            "TABLE_ID": "{{ var.json.census_bureau_acs.zcta_2021_5yr.destination_table }}",
            "SCHEMA_PATH": "{{ var.json.census_bureau_acs.zcta_2021_5yr.schema_path }}",
            "PIPELINE_NAME": "zcta_2021_5yr",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.census_bureau_acs.zcta_2021_5yr.target_gcs_path }}",
            "YEAR_REPORT": "5",
            "API_NAMING_CONVENTION": "zip%20code%20tabulation%20area",
            "GEOGRAPHY": "zcta",
            "REPORT_LEVEL": "national_level",
            "CONCAT_COL_LIST": '["zip_code_tabulation_area"]',
            "RENAME_MAPPINGS_LIST": '{"0":"name", "1":"KPI_Value", "2":"state", "3":"zip_code_tabulation_area"}',
            "INPUT_CSV_HEADERS": '["0", "1", "2", "3","KPI_Name"]',
            "DATA_DTYPES": '{"geo_id": "str", "aggregate_travel_time_to_work": "str", "amerindian_including_hispanic": "str", "amerindian_pop": "str", "armed_forces": "str", "asian_including_hispanic": "str", "asian_male_45_54": "str", "asian_male_55_64": "str", "asian_pop": "str", "associates_degree": "str", "bachelors_degree": "str", "bachelors_degree_2": "str", "bachelors_degree_or_higher_25_64": "str", "black_including_hispanic": "str", "black_male_45_54": "str", "black_male_55_64": "str", "black_pop": "str", "children": "str", "children_in_single_female_hh": "str", "civilian_labor_force": "str", "commute_10_14_mins": "str", "commute_15_19_mins": "str", "commute_20_24_mins": "str", "commute_25_29_mins": "str", "commute_30_34_mins": "str", "commute_35_39_mins": "str", "commute_35_44_mins": "str", "commute_40_44_mins": "str", "commute_45_59_mins": "str", "commute_5_9_mins": "str", "commute_60_89_mins": "str", "commute_60_more_mins": "str", "commute_90_more_mins": "str", "commute_less_10_mins": "str", "commuters_16_over": "str", "commuters_by_bus": "str", "commuters_by_car_truck_van": "str", "commuters_by_carpool": "str", "commuters_by_public_transportation": "str", "commuters_by_subway_or_elevated": "str", "commuters_drove_alone": "str", "different_house_year_ago_different_city": "str", "different_house_year_ago_same_city": "str", "dwellings_10_to_19_units": "str", "dwellings_1_units_attached": "str", "dwellings_1_units_detached": "str", "dwellings_20_to_49_units": "str", "dwellings_2_units": "str", "dwellings_3_to_4_units": "str", "dwellings_50_or_more_units": "str", "dwellings_5_to_9_units": "str", "employed_agriculture_forestry_fishing_hunting_mining": "str", "employed_arts_entertainment_recreation_accommodation_food": "str", "employed_construction": "str", "employed_education_health_social": "str", "employed_finance_insurance_real_estate": "str", "employed_information": "str", "employed_manufacturing": "str", "employed_other_services_not_public_admin": "str", "employed_pop": "str", "employed_public_administration": "str", "employed_retail_trade": "str", "employed_science_management_admin_waste": "str", "employed_transportation_warehousing_utilities": "str", "employed_wholesale_trade": "str", "families_with_young_children": "str", "family_households": "str", "father_in_labor_force_one_parent_families_with_young_children": "str", "father_one_parent_families_with_young_children": "str", "female_10_to_14": "str", "female_15_to_17": "str", "female_18_to_19": "str", "female_20": "str", "female_21": "str", "female_22_to_24": "str", "female_25_to_29": "str", "female_30_to_34": "str", "female_35_to_39": "str", "female_40_to_44": "str", "female_45_to_49": "str", "female_50_to_54": "str", "female_55_to_59": "str", "female_5_to_9": "str", "female_60_to_61": "str", "female_62_to_64": "str", "female_65_to_66": "str", "female_67_to_69": "str", "female_70_to_74": "str", "female_75_to_79": "str", "female_80_to_84": "str", "female_85_and_over": "str", "female_female_households": "str", "female_pop": "str", "female_under_5": "str", "four_more_cars": "str", "gini_index": "str", "graduate_professional_degree": "str", "group_quarters": "str", "high_school_diploma": "str", "high_school_including_ged": "str", "hispanic_any_race": "str", "hispanic_male_45_54": "str", "hispanic_male_55_64": "str", "hispanic_pop": "str", "households": "str", "households_public_asst_or_food_stamps": "str", "households_retirement_income": "str", "housing_built_1939_or_earlier": "str", "housing_built_2000_to_2004": "str", "housing_built_2005_or_later": "str", "housing_units": "str", "housing_units_renter_occupied": "str", "in_grades_1_to_4": "str", "in_grades_5_to_8": "str", "in_grades_9_to_12": "str", "in_school": "str", "in_undergrad_college": "str", "income_100000_124999": "str", "income_10000_14999": "str", "income_125000_149999": "str", "income_150000_199999": "str", "income_15000_19999": "str", "income_200000_or_more": "str", "income_20000_24999": "str", "income_25000_29999": "str", "income_30000_34999": "str", "income_35000_39999": "str", "income_40000_44999": "str", "income_45000_49999": "str", "income_50000_59999": "str", "income_60000_74999": "str", "income_75000_99999": "str", "income_less_10000": "str", "income_per_capita": "str", "less_one_year_college": "str", "less_than_high_school_graduate": "str", "male_10_to_14": "str", "male_15_to_17": "str", "male_18_to_19": "str", "male_20": "str", "male_21": "str", "male_22_to_24": "str", "male_25_to_29": "str", "male_30_to_34": "str", "male_35_to_39": "str", "male_40_to_44": "str", "male_45_64_associates_degree": "str", "male_45_64_bachelors_degree": "str", "male_45_64_grade_9_12": "str", "male_45_64_graduate_degree": "str", "male_45_64_high_school": "str", "male_45_64_less_than_9_grade": "str", "male_45_64_some_college": "str", "male_45_to_49": "str", "male_45_to_64": "str", "male_50_to_54": "str", "male_55_to_59": "str", "male_5_to_9": "str", "male_60_to_61": "str", "male_62_to_64": "str", "male_65_to_66": "str", "male_67_to_69": "str", "male_70_to_74": "str", "male_75_to_79": "str", "male_80_to_84": "str", "male_85_and_over": "str", "male_male_households": "str", "male_pop": "str", "male_under_5": "str", "management_business_sci_arts_employed": "str", "married_households": "str", "masters_degree": "str", "median_age": "str", "median_income": "str", "median_rent": "str", "median_year_structure_built": "str", "million_dollar_housing_units": "str", "mobile_homes": "str", "mortgaged_housing_units": "str", "no_car": "str", "no_cars": "str", "nonfamily_households": "str", "not_hispanic_pop": "str", "not_in_labor_force": "str", "not_us_citizen_pop": "str", "occupation_management_arts": "str", "occupation_natural_resources_construction_maintenance": "str", "occupation_production_transportation_material": "str", "occupation_sales_office": "str", "occupation_services": "str", "occupied_housing_units": "str", "one_car": "str", "one_parent_families_with_young_children": "str", "one_year_more_college": "str", "other_race_pop": "str", "owner_occupied_housing_units": "str", "owner_occupied_housing_units_lower_value_quartile": "str", "owner_occupied_housing_units_median_value": "str", "owner_occupied_housing_units_upper_value_quartile": "str", "percent_income_spent_on_rent": "str", "pop_16_over": "str", "pop_25_64": "str", "pop_25_years_over": "str", "pop_5_years_over": "str", "pop_determined_poverty_status": "str", "pop_in_labor_force": "str", "population_1_year_and_over": "str", "population_3_years_over": "str", "poverty": "str", "rent_10_to_15_percent": "str", "rent_15_to_20_percent": "str", "rent_20_to_25_percent": "str", "rent_25_to_30_percent": "str", "rent_30_to_35_percent": "str", "rent_35_to_40_percent": "str", "rent_40_to_50_percent": "str", "rent_burden_not_computed": "str", "rent_over_50_percent": "str", "rent_under_10_percent": "str", "renter_occupied_housing_units_paying_cash_median_gross_rent": "str", "sales_office_employed": "str", "some_college_and_associates_degree": "str", "speak_only_english_at_home": "str", "speak_spanish_at_home": "str", "speak_spanish_at_home_low_english": "str", "three_cars": "str", "total_pop": "str", "two_cars": "str", "two_or_more_races_pop": "str", "two_parent_families_with_young_children": "str", "two_parents_father_in_labor_force_families_with_young_children": "str", "two_parents_in_labor_force_families_with_young_children": "str", "two_parents_mother_in_labor_force_families_with_young_children": "str", "two_parents_not_in_labor_force_families_with_young_children": "str", "unemployed_pop": "str", "vacant_housing_units": "str", "vacant_housing_units_for_rent": "str", "vacant_housing_units_for_sale": "str", "walked_to_work": "str", "white_including_hispanic": "str", "white_male_45_54": "str", "white_male_55_64": "str", "white_pop": "str", "worked_at_home": "str", "workers_16_and_over": "str"}',
            "OUTPUT_CSV_HEADERS": '["geo_id", "aggregate_travel_time_to_work", "amerindian_including_hispanic", "amerindian_pop", "armed_forces", "asian_including_hispanic", "asian_male_45_54", "asian_male_55_64", "asian_pop", "associates_degree", "bachelors_degree", "bachelors_degree_2", "bachelors_degree_or_higher_25_64", "black_including_hispanic", "black_male_45_54", "black_male_55_64", "black_pop", "children", "children_in_single_female_hh", "civilian_labor_force", "commute_10_14_mins", "commute_15_19_mins", "commute_20_24_mins", "commute_25_29_mins", "commute_30_34_mins", "commute_35_39_mins", "commute_35_44_mins", "commute_40_44_mins", "commute_45_59_mins", "commute_5_9_mins", "commute_60_89_mins", "commute_60_more_mins", "commute_90_more_mins", "commute_less_10_mins", "commuters_16_over", "commuters_by_bus", "commuters_by_car_truck_van", "commuters_by_carpool", "commuters_by_public_transportation", "commuters_by_subway_or_elevated", "commuters_drove_alone", "different_house_year_ago_different_city", "different_house_year_ago_same_city", "dwellings_10_to_19_units", "dwellings_1_units_attached", "dwellings_1_units_detached", "dwellings_20_to_49_units", "dwellings_2_units", "dwellings_3_to_4_units", "dwellings_50_or_more_units", "dwellings_5_to_9_units", "employed_agriculture_forestry_fishing_hunting_mining", "employed_arts_entertainment_recreation_accommodation_food", "employed_construction", "employed_education_health_social", "employed_finance_insurance_real_estate", "employed_information", "employed_manufacturing", "employed_other_services_not_public_admin", "employed_pop", "employed_public_administration", "employed_retail_trade", "employed_science_management_admin_waste", "employed_transportation_warehousing_utilities", "employed_wholesale_trade", "families_with_young_children", "family_households", "father_in_labor_force_one_parent_families_with_young_children", "father_one_parent_families_with_young_children", "female_10_to_14", "female_15_to_17", "female_18_to_19", "female_20", "female_21", "female_22_to_24", "female_25_to_29", "female_30_to_34", "female_35_to_39", "female_40_to_44", "female_45_to_49", "female_50_to_54", "female_55_to_59", "female_5_to_9", "female_60_to_61", "female_62_to_64", "female_65_to_66", "female_67_to_69", "female_70_to_74", "female_75_to_79", "female_80_to_84", "female_85_and_over", "female_female_households", "female_pop", "female_under_5", "four_more_cars", "gini_index", "graduate_professional_degree", "group_quarters", "high_school_diploma", "high_school_including_ged", "hispanic_any_race", "hispanic_male_45_54", "hispanic_male_55_64", "hispanic_pop", "households", "households_public_asst_or_food_stamps", "households_retirement_income", "housing_built_1939_or_earlier", "housing_built_2000_to_2004", "housing_built_2005_or_later", "housing_units", "housing_units_renter_occupied", "in_grades_1_to_4", "in_grades_5_to_8", "in_grades_9_to_12", "in_school", "in_undergrad_college", "income_100000_124999", "income_10000_14999", "income_125000_149999", "income_150000_199999", "income_15000_19999", "income_200000_or_more", "income_20000_24999", "income_25000_29999", "income_30000_34999", "income_35000_39999", "income_40000_44999", "income_45000_49999", "income_50000_59999", "income_60000_74999", "income_75000_99999", "income_less_10000", "income_per_capita", "less_one_year_college", "less_than_high_school_graduate", "male_10_to_14", "male_15_to_17", "male_18_to_19", "male_20", "male_21", "male_22_to_24", "male_25_to_29", "male_30_to_34", "male_35_to_39", "male_40_to_44", "male_45_64_associates_degree", "male_45_64_bachelors_degree", "male_45_64_grade_9_12", "male_45_64_graduate_degree", "male_45_64_high_school", "male_45_64_less_than_9_grade", "male_45_64_some_college", "male_45_to_49", "male_45_to_64", "male_50_to_54", "male_55_to_59", "male_5_to_9", "male_60_to_61", "male_62_to_64", "male_65_to_66", "male_67_to_69", "male_70_to_74", "male_75_to_79", "male_80_to_84", "male_85_and_over", "male_male_households", "male_pop", "male_under_5", "management_business_sci_arts_employed", "married_households", "masters_degree", "median_age", "median_income", "median_rent", "median_year_structure_built", "million_dollar_housing_units", "mobile_homes", "mortgaged_housing_units", "no_car", "no_cars", "nonfamily_households", "not_hispanic_pop", "not_in_labor_force", "not_us_citizen_pop", "occupation_management_arts", "occupation_natural_resources_construction_maintenance", "occupation_production_transportation_material", "occupation_sales_office", "occupation_services", "occupied_housing_units", "one_car", "one_parent_families_with_young_children", "one_year_more_college", "other_race_pop", "owner_occupied_housing_units", "owner_occupied_housing_units_lower_value_quartile", "owner_occupied_housing_units_median_value", "owner_occupied_housing_units_upper_value_quartile", "percent_income_spent_on_rent", "pop_16_over", "pop_25_64", "pop_25_years_over", "pop_5_years_over", "pop_determined_poverty_status", "pop_in_labor_force", "population_1_year_and_over", "population_3_years_over", "poverty", "rent_10_to_15_percent", "rent_15_to_20_percent", "rent_20_to_25_percent", "rent_25_to_30_percent", "rent_30_to_35_percent", "rent_35_to_40_percent", "rent_40_to_50_percent", "rent_burden_not_computed", "rent_over_50_percent", "rent_under_10_percent", "renter_occupied_housing_units_paying_cash_median_gross_rent", "sales_office_employed", "some_college_and_associates_degree", "speak_only_english_at_home", "speak_spanish_at_home", "speak_spanish_at_home_low_english", "three_cars", "total_pop", "two_cars", "two_or_more_races_pop", "two_parent_families_with_young_children", "two_parents_father_in_labor_force_families_with_young_children", "two_parents_in_labor_force_families_with_young_children", "two_parents_mother_in_labor_force_families_with_young_children", "two_parents_not_in_labor_force_families_with_young_children", "unemployed_pop", "vacant_housing_units", "vacant_housing_units_for_rent", "vacant_housing_units_for_sale", "walked_to_work", "white_including_hispanic", "white_male_45_54", "white_male_55_64", "white_pop", "worked_at_home", "workers_16_and_over"]',
        },
        resources={"request_ephemeral_storage": "8G", "request_cpu": "1"},
    )
    delete_cluster = kubernetes_engine.GKEDeleteClusterOperator(
        task_id="delete_cluster",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        name="census_bureau_acs",
    )

    (
        create_cluster
        >> [
            cbsa_2019_1yr,
            cbsa_2020_1yr,
            cbsa_2021_1yr,
            cbsa_2019_5yr,
            cbsa_2020_5yr,
            cbsa_2021_5yr,
            censustract_2019_5yr,
            censustract_2020_5yr,
            censustract_2021_5yr,
            congressionaldistrict_2019_1yr,
            congressionaldistrict_2020_1yr,
            congressionaldistrict_2021_1yr,
            congressionaldistrict_2019_5yr,
            congressionaldistrict_2020_5yr,
            congressionaldistrict_2021_5yr,
            county_2019_1yr,
            county_2020_1yr,
            county_2021_1yr,
            county_2019_5yr,
            county_2020_5yr,
            county_2021_5yr,
            place_2019_1yr,
            place_2020_1yr,
            place_2021_1yr,
            place_2019_5yr,
            place_2020_5yr,
            place_2021_5yr,
            puma_2019_1yr,
            puma_2020_1yr,
            puma_2021_1yr,
            puma_2019_5yr,
            puma_2020_5yr,
            puma_2021_5yr,
            schooldistrictelementary_2016_1yr,
            schooldistrictelementary_2017_1yr,
            schooldistrictelementary_2018_1yr,
            schooldistrictelementary_2019_1yr,
            schooldistrictelementary_2020_1yr,
            schooldistrictelementary_2021_1yr,
            schooldistrictelementary_2016_5yr,
            schooldistrictelementary_2017_5yr,
            schooldistrictelementary_2018_5yr,
            schooldistrictelementary_2019_5yr,
            schooldistrictelementary_2020_5yr,
            schooldistrictelementary_2021_5yr,
            schooldistrictsecondary_2016_1yr,
            schooldistrictsecondary_2017_1yr,
            schooldistrictsecondary_2018_1yr,
            schooldistrictsecondary_2019_1yr,
            schooldistrictsecondary_2020_1yr,
            schooldistrictsecondary_2021_1yr,
            schooldistrictsecondary_2016_5yr,
            schooldistrictsecondary_2017_5yr,
            schooldistrictsecondary_2018_5yr,
            schooldistrictsecondary_2019_5yr,
            schooldistrictsecondary_2020_5yr,
            schooldistrictsecondary_2021_5yr,
            schooldistrictunified_2016_1yr,
            schooldistrictunified_2017_1yr,
            schooldistrictunified_2018_1yr,
            schooldistrictunified_2019_1yr,
            schooldistrictunified_2020_1yr,
            schooldistrictunified_2021_1yr,
            schooldistrictunified_2016_5yr,
            schooldistrictunified_2017_5yr,
            schooldistrictunified_2018_5yr,
            schooldistrictunified_2019_5yr,
            schooldistrictunified_2020_5yr,
            schooldistrictunified_2021_5yr,
            state_2019_1yr,
            state_2020_1yr,
            state_2021_1yr,
            state_2019_5yr,
            state_2020_5yr,
            state_2021_5yr,
            zcta_2019_5yr,
            zcta_2020_5yr,
            zcta_2021_5yr,
        ]
        >> delete_cluster
    )
